//*********************************************//
//                e-Bot (4x4)                  //
//                  MAZEBOT                    //
//                                             //
//  This enables a bot to avoid obstacles and  //
//          may also solve mazes.              //
//               Codes by:                     //
//        e-Gizmo Mechatronix Central          //
//         Taft, Manila, Philippines           //
//           http://www.egizmo.com             //
//                May 15,2016                  //
//*********************************************//

// INITIALIZATION:
#define MOTOR_1_DIRECTION         8       // Motor control HIGH = FWD, LOW = BWD
#define MOTOR_1_SPEED             9       // Speed input from 0 to 255 (left motor)
#define MOTOR_2_SPEED            10       // Speed input from 0 to 255 (right motor)
#define MOTOR_2_DIRECTION        11       // Motor control HIGH = FWD, LOW = BWD

//LIBRARIES
#include <NewPing.h>        
#include <SM.h>

//ULTRASONIC PINS
#define TRIGGER_PIN  12
#define ECHO_PIN    3

//MAXIMUM DISTANCE WE WANT TO PING IN CM (TO 500 CM)
#define MAX_DISTANCE 100  

//SERVO PIN
#define SERVO_PIN    6

//OPPONENTS "FOUND" INDICATOR
#define PIN13_LED    13

//----( "OPPONENTS" DIRECTIONS (index into TargetArray )---------
#define TARGET_FOUND_ANY     0   // Values will be "true" or "false"
#define TARGET_LEFT          1
#define TARGET_LEFT_CENTER   2
#define TARGET_CENTER        3
#define TARGET_RIGHT_CENTER  4
#define TARGET_RIGHT         5
#define TARGET_ARRAY_SIZE    6

////OPPONENTS "FOUND" DISTANCE 
#define TARGET_TOO_CLOSE     30

NewPing sonar(TRIGGER_PIN, ECHO_PIN, MAX_DISTANCE); // Set pins and maximum distance.
SM eBotGo(eBotStart);//create simple statemachine

/*-----( Declare Variables )-----*/
unsigned int uS;  // Result of a ping: MicroSeconds
unsigned int cm;  // Distance calculated for ping (0 = outside set distance range)
unsigned int cm_now; // For test

int TargetArray[TARGET_ARRAY_SIZE];  // Holds the directions a Target was found in
int DirectionsToLook = 4;  // For LookAround()
int ServoDirectionData[4] = { 1600, 2500, 600, 1600};

// INPUTS AND OUTPUTS
void setup()  
{
  Serial.begin(115200);
  pinMode(MOTOR_1_DIRECTION, OUTPUT);
  pinMode(MOTOR_1_SPEED, OUTPUT);
  pinMode(MOTOR_2_SPEED, OUTPUT);
  pinMode(MOTOR_2_DIRECTION, OUTPUT);
  pinMode(SERVO_PIN,OUTPUT);  
  pinMode(PIN13_LED,OUTPUT);    
}
// MAIN PROGRAM
void loop()   
{
  EXEC(eBotGo);                   //run statemachine
  delay(100);
}
// MOTOR CONTROL FUNCTION
void FORWARD_SLOW()
{
  digitalWrite(MOTOR_1_DIRECTION, HIGH); 
  digitalWrite(MOTOR_1_SPEED, 128);
  digitalWrite(MOTOR_2_SPEED, 128);
  digitalWrite(MOTOR_2_DIRECTION, HIGH);
}
/*---------------------------*/
void FORWARD_MEDIUM()
{
  digitalWrite(MOTOR_1_DIRECTION, HIGH); 
  digitalWrite(MOTOR_1_SPEED, 200);
  digitalWrite(MOTOR_2_SPEED, 200);
  digitalWrite(MOTOR_2_DIRECTION, HIGH);
  ; 
}
/*---------------------------*/
void FORWARD_FAST()
{
  digitalWrite(MOTOR_1_DIRECTION, HIGH); 
  digitalWrite(MOTOR_1_SPEED, HIGH);
  digitalWrite(MOTOR_2_SPEED, HIGH);
  digitalWrite(MOTOR_2_DIRECTION, HIGH);
}
/*---------------------------*/
void BACKWARD_SLOW(int runspeed)
{
  digitalWrite(MOTOR_1_DIRECTION, LOW); 
  digitalWrite(MOTOR_1_SPEED, runspeed);
  digitalWrite(MOTOR_2_SPEED, runspeed);
  digitalWrite(MOTOR_2_DIRECTION, LOW);
  delay(1000);
}
/*---------------------------*/
void BACKWARD_MEDIUM(int runspeed)
{
  digitalWrite(MOTOR_1_DIRECTION, LOW); 
  digitalWrite(MOTOR_1_SPEED, runspeed);
  digitalWrite(MOTOR_2_SPEED, runspeed);
  digitalWrite(MOTOR_2_DIRECTION, LOW);
}
/*---------------------------*/
void BACKWARD_FAST(int runspeed)
{
  digitalWrite(MOTOR_1_DIRECTION, LOW); 
  digitalWrite(MOTOR_1_SPEED, runspeed);
  digitalWrite(MOTOR_2_SPEED, runspeed);
  digitalWrite(MOTOR_2_DIRECTION, LOW);
}
/*---------------------------*/
void STOP()
{
  digitalWrite(MOTOR_1_DIRECTION, HIGH); 
  digitalWrite(MOTOR_1_SPEED, LOW);
  digitalWrite(MOTOR_2_SPEED, LOW);
  digitalWrite(MOTOR_2_DIRECTION, HIGH); 
}
/*---------------------------*/
void SPIN_LEFT(int runspeed)
{
  digitalWrite(MOTOR_1_DIRECTION, HIGH); 
  digitalWrite(MOTOR_1_SPEED, HIGH);
  digitalWrite(MOTOR_2_SPEED, HIGH);
  digitalWrite(MOTOR_2_DIRECTION, LOW);
  delay(runspeed);
}
/*---------------------------*/
void SPIN_RIGHT(int runspeed)
{
  digitalWrite(MOTOR_1_DIRECTION, LOW); 
  digitalWrite(MOTOR_1_SPEED, HIGH);
  digitalWrite(MOTOR_2_SPEED, HIGH);
  digitalWrite(MOTOR_2_DIRECTION, HIGH);
  delay(runspeed);
}
/*---------------------------*/
unsigned int PingBlink()
{
  uS = sonar.ping();               // Send ping, get ping time in microseconds (uS).
  cm = uS / US_ROUNDTRIP_CM;       // Convert ping time to distance in cm 
  Serial.print("  cm = ");   
  Serial.print(cm,DEC);  
  if ((cm < 80) && (cm != 0))
  {
    return(cm);
  }
  else
  {
    return(100);                   // No Valid Distance
  }
}
/*---------------------------*/
void PointServo(int ServoAngle)
{
  for (int i=0;i<20;i++)           // Send the pulse 10 times 
  {
    digitalWrite(SERVO_PIN,HIGH);
    delayMicroseconds(ServoAngle);
    digitalWrite(SERVO_PIN,LOW);
    delay(20);
  }
}
/*---------------------------*/
void LookAround()                 // Sets next state if Target Found
{
  for(int Direction = 0; Direction < DirectionsToLook ; Direction ++)
  {
    Serial.print("DIRECTION = ");     
    Serial.print(Direction,DEC);
    PointServo(ServoDirectionData[Direction]);      // Get servo pulse width from array
    delay(100);
    cm_now = PingBlink();                           // Read the Ultrasonic distance

    Serial.print(" cm_now = ");     
    Serial.println(cm_now,DEC);    
    if (cm_now < TARGET_TOO_CLOSE) digitalWrite(PIN13_LED,HIGH);    
    STOP();
    if (cm_now < 80)
    {
      TargetArray[TARGET_FOUND_ANY ] = true;
    }
    else TargetArray[TARGET_FOUND_ANY ] = false;

    if ((cm_now < TARGET_TOO_CLOSE) && (Direction == 0)) //LEFT
    {
      TargetArray[TARGET_LEFT ] = true;
      Serial.println("e-Bot_found_obstacle_on_Left");  
    }    
    if ((cm_now < TARGET_TOO_CLOSE) && (Direction == 1)) //Center
    {
      TargetArray[TARGET_CENTER ] = true;
      Serial.println("e-Bot_found_obstacle_on_Center");       
    }    
    if ((cm_now < TARGET_TOO_CLOSE) && (Direction == 2)) //RIGHT
    {
      TargetArray[TARGET_RIGHT ] = true;
      Serial.println("e-Bot_found_obstacle_on_Right");       
    }    
  }
}
/**************************( STATE MACHINE FUNCTIONS )******************************/
State eBotStart()
{
  Serial.println("e-Bot_Start");
  digitalWrite(PIN13_LED,LOW);    // LED Means Target Too Close
  FORWARD_FAST();  // Start moving forward
  delay(25);
  FORWARD_SLOW();  // Start moving forward
  delay(600);
  eBotGo.Set(eBotSTOPtoLook);    
}
/**********************************************************************************/
State eBotSTOPtoLook()
{
  Serial.println("e-Bot_stop_look");
  for (int i = 0; i < TARGET_ARRAY_SIZE; i++) TargetArray[i] = false;
  STOP();                        // Start moving forward
  LookAround();                  // Ping Ultrasonic in different directions, Set TargetArray
  if      (TargetArray[TARGET_CENTER ] == true)  eBotGo.Set(eboTargetCenter);
  else if (TargetArray[TARGET_LEFT ]   == true)  eBotGo.Set(eboTargetLeft);
  else if (TargetArray[TARGET_RIGHT ]  == true)  eBotGo.Set(eboTargetRight);
  else     eBotGo.Set(eBotStart);  
}
/**********************************************************************************/
State eboTargetLeft()
{
  Serial.println("e-Bot_found_target_on_left");
  BACKWARD_SLOW(50);
  SPIN_RIGHT(500); 
  eBotGo.Set(eBotStart);  
}
/**********************************************************************************/
State eboTargetCenter()  
{
  Serial.println("e-Bot_found_target_on_left_Center");  
  STOP();
  BACKWARD_SLOW(50);  
  SPIN_RIGHT(500);
  eBotGo.Set(eBotStart); 
}
/**********************************************************************************/
State eboTargetRight()
{
  Serial.println("e-Bot_found_target_on_left_Right");   
  BACKWARD_SLOW(50);
  SPIN_LEFT(500); 
  eBotGo.Set(eBotStart); 
}
// PROGRAM ENDS AND LOOP BACK FOREVER                                                                                                                                                   // PARZ
/**********************************************************************************/


